/*
** Vinco_LCD.c
**
** Copyright  2011 Future Devices International Limited
**
** C Source file for Vinculum II test applications
** Main module
**
** Author: P. Rybak - FTDI
** Project: Vinculum II
** Module: Vinculum II Applications
** Requires: VOS, GPIO
** Comments:
**
** History:
**  1  Initial version
**
*/

#include "vos.h"
#include "Vinco_LCD.h"
#include "GPIO.h"


// VOS TCB pointers for threads
vos_tcb_t        *tcbFirmware;

// forward declarations
void firmware(void);


// external declarations

/* main
*
* This is the main application called by VOS at startup. It is responsible
* for device and thread initialisation.
* NOTE: this will be renamed to main() when the program loader is integrated.
*
*
*
*
*
*
*/
void main(void)
{
    // GPIO context structure
    gpio_context_t gpioCtx;

    // call VOS initialisation routines
    vos_init(10, VOS_TICK_INTERVAL, NUMBER_OF_DEVICES);
    vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);

    if (vos_get_package_type() == VINCULUM_II_64_PIN)
    {
        // GPIO port A bit 0 to pin 51
        vos_iomux_define_output(51,IOMUX_OUT_GPIO_PORT_A_0); // DB4
        // GPIO port A bit 1 to pin 52
        vos_iomux_define_output(52,IOMUX_OUT_GPIO_PORT_A_1); // DB5
        // GPIO port A bit 2 to pin 55
        vos_iomux_define_output(55,IOMUX_OUT_GPIO_PORT_A_2); // DB6
        // GPIO port A bit 3 to pin 56
        vos_iomux_define_output(56,IOMUX_OUT_GPIO_PORT_A_3); // DB7
        // GPIO port A bit 4 to pin 57
        vos_iomux_define_output(57,IOMUX_OUT_GPIO_PORT_A_4); // RS
        // GPIO port A bit 5 to pin 58
        vos_iomux_define_output(58,IOMUX_OUT_GPIO_PORT_A_5); // E
    }

    // initialise device drivers
    // TODO: call initialisation routines for included device drivers
    gpioCtx.port_identifier = GPIO_PORT_A;
    gpio_init(VOS_DEV_GPIO,&gpioCtx);

    // create threads for firmware application (no parameters)
    tcbFirmware = vos_create_thread(29, SIZEOF_THREAD_MEMORY, firmware, 0);
    // start VOS scheduler
    vos_start_scheduler();

main_loop:
    goto main_loop;
}

void write_lcd_cmd(VOS_HANDLE hLCD, unsigned char byte)
{
    unsigned char cmd;
    // Write High nibble data to LCD
    cmd = (((byte >> 4) &0x0F) | lcd_e);
    cmd = (cmd &(~lcd_rs)); // Select Registers
    vos_dev_write(hLCD,&cmd,1,NULL);
    // Toggle 'E' pin
    cmd &= (~lcd_e);
    vos_dev_write(hLCD,&cmd,1,NULL);
    // Write Low nibble data to LCD
    cmd = ((byte &0x0F) | lcd_e);
    cmd = (cmd &(~lcd_rs)); // Select Registers
    vos_dev_write(hLCD,&cmd,1,NULL);
    // Toggle 'E' pin
    cmd &= (~lcd_e);
    vos_dev_write(hLCD,&cmd,1,NULL);
    vos_delay_msecs(1);

}

void write_lcd_data(VOS_HANDLE hLCD, unsigned char byte)
{
    unsigned char cmd;

    // Write High nibble data to LCD
    cmd = (((byte >> 4)&0x0F) | lcd_rs);
    cmd = (cmd | lcd_e); // Select DDRAM
    vos_dev_write(hLCD,&cmd,1,NULL);
    // Toggle 'E' pin
    cmd &= (~lcd_e);
    vos_dev_write(hLCD,&cmd,1,NULL);
    // Write Low nibble data to LCD
    cmd = ((byte & 0x0F) | lcd_rs);
    cmd = (cmd | lcd_e); // Select DDRAM
    vos_dev_write(hLCD,&cmd,1,NULL);
    // Toggle 'E' pin
    cmd &= (~lcd_e);
    vos_dev_write(hLCD,&cmd,1,NULL);
    vos_delay_msecs(1);

}
void write_lcd_str(VOS_HANDLE hLCD, unsigned char *str)
{
    while(*str != '\0')
    {
         write_lcd_data(hLCD, *str);
         ++str;
    }
}

void lcd_ini(VOS_HANDLE hLCD)
{
    vos_delay_msecs(100);
    // Send Reset command
    write_lcd_cmd(hLCD, 0x03);
    vos_delay_msecs(2);
    // Send Function Set
    write_lcd_cmd(hLCD, 0x28);
    vos_delay_msecs(2);
    write_lcd_cmd(hLCD, 0x28);
    vos_delay_msecs(2);
    // Send Display control command
    write_lcd_cmd(hLCD, 0x0C);
    vos_delay_msecs(2);
    // Send Display Clear command
    write_lcd_cmd(hLCD, 0x01);
    vos_delay_msecs(2);
    // Send Entry Mode Set command
    write_lcd_cmd(hLCD, 0x06);
    vos_delay_msecs(2);
}

/* firmware
*
* This is the thread created by the main application.
*
*
*
*
*
*
*/
void firmware(void)
{
    // VOS handles for opened devices
    VOS_HANDLE hGpio;

    // GPIO IOCTL request block
    gpio_ioctl_cb_t gpio_iocb;

    // string to display (include space for terminating NULL)
    unsigned char *lcd_str = "Vinco";

    // find and open GPIO device port A
    hGpio = vos_dev_open(VOS_DEV_GPIO);

    gpio_iocb.ioctl_code = VOS_IOCTL_GPIO_SET_MASK;
    gpio_iocb.value = 0xFF;    // set all as output
    vos_dev_ioctl(hGpio, &gpio_iocb);

    lcd_ini(hGpio);

    // thread loop
    do
    {
        // Send Display Clear command
        write_lcd_cmd(hGpio, 0x01);
        vos_delay_msecs(2);

        // Set 1-st line address
        write_lcd_cmd(hGpio, (0x01 | 0x80));
        // Set string for the 1-st line
        lcd_str = "*** Vinco ***";
        // Send string to LCD
        write_lcd_str(hGpio, lcd_str);

        // Set 2-nd line address
        write_lcd_cmd(hGpio, (0x40 | 0x80));
        // Set string for 2-nd line
        lcd_str = "www.ftdichip.com";
        // Send string to LCD
        write_lcd_str(hGpio, lcd_str);

        // Wait a while
        vos_delay_msecs(2000);

        // Send Display Clear command
        write_lcd_cmd(hGpio, 0x01);
        vos_delay_msecs(2);

        // Set 1-st line address
        write_lcd_cmd(hGpio, (0x01 | 0x80));
        // Set string for the 1-st line
        lcd_str = "The Best Board";
        write_lcd_str(hGpio, lcd_str);

        // Set 2-nd line address
        write_lcd_cmd(hGpio, (0x45 | 0x80));
        // Set string for 2-nd line
        lcd_str = "Ever!!!";
        write_lcd_str(hGpio, lcd_str);

        vos_delay_msecs(2000);
    } while (1);
}

